@@ -8,6 +8,10 @@
 #include "picinfomodel.h"
 #include "imagetools_plugin.h"
 
+#ifndef Q_OS_ANDROID
+#include <ocs.h>
+#endif
+
 void ImageToolsPlugin::registerTypes(const char *uri)
 {
     #if defined(Q_OS_ANDROID)
@@ -19,6 +23,12 @@ void ImageToolsPlugin::registerTypes(const char *uri)
     
     qmlRegisterType<PicInfoModel>(uri, 1, 3, "PicInfoModel");
     qmlRegisterType(resolveFileUrl(QStringLiteral("ImageInfoDialog.qml")), uri, 1, 3, "ImageInfoDialog");     
-    qmlRegisterType(resolveFileUrl(QStringLiteral("MetadataEditor.qml")), uri, 1, 3, "MetadataEditor");     
+    qmlRegisterType(resolveFileUrl(QStringLiteral("MetadataEditor.qml")), uri, 1, 3, "MetadataEditor");
+
+#ifndef Q_OS_ANDROID
+    qmlRegisterType<OCS>(uri, 1, 3, "OCR");
+    qmlRegisterType(resolveFileUrl(QStringLiteral("image2text/OCRPage.qml")), uri, 1, 3, "OCRPage");
+#endif
+
 }
 
